#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdbool.h>
#include <assert.h>
#include "crypto_aead.h"

#ifndef KEY_LENGTH
#define KEY_LENGTH 16
#endif

#ifndef NSEC_LENGTH
#define NSEC_LENGTH 0
#endif

#ifndef NPUB_LENGTH
#define NPUB_LENGTH 0
#endif

unsigned char *c = NULL;
unsigned long long clen = 0;
unsigned char *m = NULL;
unsigned long long mlen = 0;
unsigned char *ad = NULL;
unsigned long long adlen = 0;
unsigned char *nsec = NULL;

unsigned long long nslen = NSEC_LENGTH;
unsigned char *npub = NULL;
unsigned long long nplen = NPUB_LENGTH;
unsigned char *k = NULL;
unsigned long long klen = KEY_LENGTH;

static void read_variable(unsigned char **target, unsigned long long *lenp) {
    if (*target != NULL) {
        free(*target);
    }
    
    uint32_t len;
    if (1 != fread(&len, sizeof(len), 1, stdin)) {
        fprintf(stderr, "ERROR: couldn't read length\r\n");
        exit(1);
    }

    *lenp = len;
    if (*lenp == 0) {
        *target = NULL;
        return;
    }

    *target = malloc(*lenp);

    if (NULL == *target) {
        fprintf(stderr, "ERROR: couldn't malloc %llu bytes\r\n", *lenp);
        exit(2);
    }

    if (1 != fread(*target, *lenp, 1, stdin)) {
        fprintf(stderr, "ERROR: didn't read %llu bytes of data\r\n", *lenp);
        exit(1);
    }
}


static void write_variable(unsigned char *target, unsigned long long len) {
    uint32_t olen = len;
    if (1 != fwrite(&olen, sizeof(olen), 1, stdout)) {
        fprintf(stderr, "ERROR: didn't write length\r\n", len);
        exit(1);
    }
    if (1 != fwrite(target, olen, 1, stdout)) {
        fprintf(stderr, "ERROR: didn't write %llu bytes of data\r\n", len);
        exit(1);
    }
}


static bool is_ready() {
    return !(k == NULL || m == NULL || c == NULL
            || ad == NULL || nsec == NULL || npub == NULL);
}


int main() {
    int res;
    int8_t action;

    setvbuf(stdout, NULL, _IONBF, 0);
    printf("Hello, World!\n");
    while (1) {
        if (1 != fread(&action, sizeof(action), 1, stdin))
            return 1;

        switch (action) {
            case 'c': read_variable(&c, &clen); break;
            case 'm': read_variable(&m, &mlen); break;
            case 'a': read_variable(&ad, &adlen); break;
            case 'k': read_variable(&k, &klen); break;

            case 's': read_variable(&nsec, &nslen); break;
            case 'p': read_variable(&npub, &nplen); break;

            case 'C': write_variable(c, clen); break;
            case 'M': write_variable(m, mlen); break;
            case 'A': write_variable(ad, adlen); break;
            case 'K': write_variable(k, klen); break;
            case 'S': write_variable(nsec, nslen); break;
            case 'P': write_variable(npub, nplen); break;


            case 'e':
                if (!is_ready()) {
                    return 3;
                }
                fprintf(stderr, "Starting encryption\r\n");
                res = crypto_aead_encrypt(c, &clen,
                        m, mlen, ad, adlen, nsec, npub, k);
                fprintf(stderr, "encryption finished\r\n");
                break;

            case 'd':
                if (!is_ready()) {
                    return 3;
                }
                fprintf(stderr, "Starting decryption\r\n");
                res = crypto_aead_decrypt(m, &mlen,
                        nsec, c, clen, ad, adlen, npub, k);
                fprintf(stderr, "decryption finished\r\n");
                break;

            default:
                fprintf(stderr, "ERROR: unexpected action 0x%02x\r\n", action);
                return 2;
        }
        
    }

}

